#include <cnoid/SimpleController>
#include <cnoid/Camera>
#include <cnoid/RangeCamera>

#include <iostream>

#include <pcl/io/pcd_io.h>
#include <pcl/point_types.h>

#include <cnoid/Joystick>

using namespace cnoid;
using namespace std;

class RangeCameraController : public SimpleController
{
    RangeCamera* RangeCameraSample;
    std::ostream* os;
    SimpleControllerIO* io;
    Joystick joystick;
    bool PrevButtonState;
public:
    virtual bool initialize(SimpleControllerIO* io) override
    {
        cout<<"start initalize"<<endl;

        this->io = io;
        os = &io->os();

        RangeCameraSample = io->body()->findDevice<RangeCamera>("RangeCameraSample");
        io->enableInput(RangeCameraSample);

        return true;
    }
    virtual bool start() override
    {
        cout<<"start start"<<endl;
        return true;
    }
    virtual bool control() override
{
        joystick.readCurrentState();
        bool buttonState = joystick.getButtonState(cnoid::Joystick::A_BUTTON);
        if(buttonState && !PrevButtonState){
            cout<<"push A_BUTTON"<<endl;
            getPointCloud();
        
        }
        PrevButtonState = buttonState;
        return true;
    }
    void getPointCloud()
    {
        //  ݂̃V[̉摜擾
        const Image& RangeImage = RangeCameraSample->constImage();
        //  ݂̃V[̉摜ۑ
        RangeImage.save("pointcloud.png");
        //  摜̍Ɖ
        const int width  = RangeImage.width();
        const int height = RangeImage.height();
        //  Ff[^擾
        const unsigned char* pixels = RangeImage.pixels();
        
        // Point Cloud̕ϐ錾
        pcl::PointCloud<pcl::PointXYZRGB> cloud;
        // Point Cloud̏
        cloud.width    = width;
        cloud.height   = height;
        cloud.is_dense = false;
        cloud.points.resize(cloud.width * cloud.height);

        std::size_t i = 0;

        //  Point CloudɊe_̒liWAFji[
        for(const auto& e: RangeCameraSample->constPoints()) {

            // X,Y,Zi[
            cloud[i].x = e(0);
            cloud[i].y = e(1);
            cloud[i].z = e(2);
            //  FiR,G,Bji[
            cloud[i].r = pixels[3*i + 0];
            cloud[i].g = pixels[3*i + 1];
            cloud[i].b = pixels[3*i + 2];
            ++i;
           
        }
        //_Qf[^ۑ
        pcl::io::savePCDFileBinaryCompressed ("pointcloud.pcd", cloud);
        cout<<"save pointcloud.pcd"<<endl;
    }

};

CNOID_IMPLEMENT_SIMPLE_CONTROLLER_FACTORY(RangeCameraController)
